﻿<?php

class XmlHandler {
	
	private $_file = "";
	private $_database;
	private $_seatCount = 16;
	
	public function __construct($file, $seatCount = 16) 
	{
		$this->_file = $file;
		$this->seatCount = $seatCount;
	}
	

	/**
	 * Adds a booking on a particular date
	 *
	 * @param string $date The date
	 * @param array $seats The seats
	 * @param string $surname The surname
	 * @param string $prename The prename
	 * @param string $contact The contact
	 * @param string $comment The comment
	 * @return SimpleXmlElement Returns an xml structure
	 *
	 */		
	public function addBooking($date, $seats, $surname, $prename, $contact, $comment = "")
	{
		if (!is_array($seats))
		{
			return $this->createError(10, "Ungültiger Sitzplatzparameter");	
		}
		
		$date = $this->normalizeDate($date);
		
		// Check validity of the seat id's
		foreach($seats as $seat) 
		{
			if (!is_numeric($seat) || $seat <= 0 || $seat > $this->_seatCount)
			{
				return $this->createError(11, "Ungültiger Sitzplatz: ".$seat);	
			}	
		}
		
		// Check the availability
		if (!$this->checkAvailability($date, $seats))
		{
			return $this->createError(12, "Mindestens ein Sitzplatz ist an diesem Tag (".$date.") nicht mehr verfügbar.");	
		}
		
		
		// Add the entry (Table and detail)
		$xmlSeats = $this->getBookingOfTheDay($date);
		
		if (!$xmlSeats)
		{
			$xmlSeats = $this->_database->table->addChild('seats');
			$xmlSeats->addAttribute('date', $date);
		}
		
		$bookingDetailId = $this->_database->booking->detail->count() + 1;
		
		// Add the seats
		foreach($seats as $seat)
		{
			$xmlSeat = $xmlSeats->addChild('seat');
			$xmlSeat->addAttribute('id', $seat);
			$xmlSeat->addAttribute('booking', $bookingDetailId);
		}
		
		// Add the booking detail
		$xmlBookingDetail = $this->_database->booking->addChild('detail', $comment);
		
		// Save the xml
		$xmlBookingDetail->addAttribute('id', $bookingDetailId);
		$xmlBookingDetail->addAttribute('surname', $surname);
		$xmlBookingDetail->addAttribute('prename', $prename);
		$xmlBookingDetail->addAttribute('contact', $contact);
		
		// Save the file (Nice formatted thx to dom ;))
		file_put_contents($this->_file, $this->_database->asXML());
		
		// Create the answer and return it
		$answer = $this->createAnswer();		
		$answer->addChild('result', 'true');
				
		return $answer->asXML();
	}
	
	
	/**
	 * Opens the database and check the integrity of it
	 *
	 * @return bool Returns true on success; otherwise false
	 *
	 */	
	public function checkDatabase()
	{
		if (!file_exists($this->_file)) return false;
					
		$this->_database = simplexml_load_file($this->_file);
		
		return $this->_database != null;
	}
	
	
	/**
	 * Gets the booking of a particular date
	 *
	 * @param string $date The date
	 * @return SimpleXmlElement Returns an xml structure with the bookings at this date
	 *
	 */	
	public function getBookings($date) 
	{
		$xml = $this->createAnswer();		
		$bookings = $xml->addChild('bookings');
		$bookings->addAttribute('date', $date);
		
		$date = $this->normalizeDate($date);
		
		$seats = array();
		
		// Fillup with free seats
		for ($value = 1; $value <= 16; $value++)
		{
			$seats[$value] = false;
		}
			
		// Lookup for all the booked seats on this day
		$booking = $this->getBookingOfTheDay($date);
		if ($booking)
		{
			foreach($booking->seat as $seat)
			{
				$seats[(int)$seat->attributes()->id[0]] = true;
			}
		}
		
		// Create xml structure
		for ($value = 1; $value <= 16; $value++)
		{
			$seat = $bookings->addChild('seat');
			$seat->addAttribute('id', $value);
			$seat->addAttribute('booked', $seats[$value] ? 'true' : 'false');
		}	
		
		return $xml->asXML();
	}
	
	/**
	 * Gets the booking states for s specific timespan
	 *
	 * @param string $start The start date
	 * @param string $end The end date
	 * @return SimpleXmlElement Returns an xml structure with the bookings during the timespan
	 *
	 */	
	public function getState($start, $end) 
	{
		$xml = $this->createAnswer();		
		$xmlBookings = $xml->addChild('bookings');
		
		$starttime = strtotime($start);
		$endtime = strtotime($end);

		while($starttime < $endtime)
		{
			$currentDate = date("d.m.Y", $starttime);	
			$booking = $this->getBookingOfTheDay($currentDate);
			
			if ($booking)
			{			
				foreach($booking->seat as $seat)
				{
					$seats[(int)$seat->attributes()->id[0]] = true;
				}
				
				$xmlDate = $xmlBookings->addChild('date');
				$xmlDate->addAttribute('date', $currentDate);
				
				$bookings = array();				
				
				// Group seats by booking detail
				foreach($booking->seat as $seat)
				{
					$id = (int)$seat->attributes()->id[0];
					$bookingId = (int)$seat->attributes()->booking[0];
					$details = null;
					
					foreach($this->_database->booking->detail as $detail)
					{
						if ((int)$detail->attributes()->id[0] == $bookingId) 
						{
							$details = $detail;
							break;	
						}
					}
					
					if ($details == null)					
					{
						return $this->createError(13, "Ein Buchungsdetail wurde nicht gefunden für den Sitzplatz ".$id);	
					}
					
					if (!array_key_exists($bookingId, $bookings)) $bookings[$bookingId] = array(0 => null, 1 => array());					
					$bookings[$bookingId][0] = $details;
					
					array_push($bookings[$bookingId][1], $id);
				}
								
				foreach($bookings as $booking)
				{
					$xmlBooking = $xmlDate->addChild("booking");
					
					// Add result attributes
					$xmlBooking->addAttribute('surname', (string)$booking[0]->attributes()->surname[0]);
					$xmlBooking->addAttribute('prename', (string)$booking[0]->attributes()->prename[0]);
					$xmlBooking->addAttribute('contact', (string)$booking[0]->attributes()->contact[0]);
					
					$xmlComment = $xmlBooking->addChild('comment', (string)$booking[0]);
					
					// Add seats
					asort($booking[1]);
					foreach($booking[1] as $seat)
					{
						$xmlSeat = $xmlBooking->addChild('seat'); 
						$xmlSeat->addAttribute('id', $seat);
					}	
				}			
			}
			
			// Add a day
			$starttime += 86400;	
		}
		
		
		return $xml->asXML();
	}
	
	
	/**
	 * Creates an error xml
	 *
	 * @param int $number The error number
	 * @param string $message The message
	 * @return SimpleXmlElement Returns an xml error object
	 *
	 */	
	public function createError($number, $message) 
	{		
		$xml = $this->createAnswer();
		
		$xml->addChild('result', 'false');
		$xmlError = $xml->addChild('error', $message);		
		$xmlError->addAttribute('number', $number);
		
		return $xml->asXML();
	}
	
	
	/**
	 * Checks the validity of a date string
	 *
	 * @param string $date The date (Format: d.m.Y)
	 * @return bool Returns true if date the matches the format; otherwise false
	 *
	 */	
	public function validateDate($date)
	{
		$newDate = DateTime::createFromFormat("d.m.Y", $date);
		
		$parts = preg_split('/[.]/', $date);
		
		if (count($parts) != 3) return false;
		
		// Add the leading zero
		$date = $this->normalizeDate($date);
						
		return $newDate && $newDate->format("d.m.Y") == $date;
	}
	
	
	/**
	 * Normalizes a date string
	 *
	 * @param string $date The date
	 * @return string Returns the normalized date string
	 *
	 */	
	private function normalizeDate($date) 
	{		
		$parts = preg_split('/[.]/', $date);
		
		return ((substr($parts[0], 0, 1) == '0' || strlen($parts[0]) == 2)? $parts[0] : '0'.$parts[0]).".".
			((substr($parts[1], 0, 1) == '0' || strlen($parts[1]) == 2) ? $parts[1] : '0'.$parts[1]).".".
			$parts[2];
	}
	
	
	/**
	 * Creates a answer xml document
	 *
	 * @return SimpleXmlElement Returns an xml document
	 *
	 */	
	private function createAnswer()
	{
		$xml = new SimpleXMLElement('<?xml version="1.0" encoding="utf-8"?><answer></answer>');
		$xml->addAttribute('datetime', date('d.m.Y H:i:s'));
		
		return $xml;
	}
	
	
	/**
	 * Checks if the seats are available at a particular date
	 *
	 * @param string $date The date
	 * @param array $seats The seats
	 * @return bool Returns true if the seats are available; otherwise false
	 *
	 */		
	private function checkAvailability($date, $seats)
	{
		$booking = $this->getBookingOfTheDay($date);
		
		if ($booking)
		{
			foreach($booking->seat as $seat)
			{
				if (in_array($seat->attributes()->id[0], $seats))
				{
					return false;
				}
			}	
		}
		
		return true;
	}
	
	
	/**
	 * Returns the booking of a particular date
	 *
	 * @param string $date The date
	 * @return SimpleXmlElement Returns the booking if it exists; otherwise null
	 *
	 */	
	private function getBookingOfTheDay($date)
	{
		// Lookup for all the booked seats on this day
		foreach($this->_database->table->seats as $booking)
		{
			if ($booking->attributes()->date[0] == $date)
			{				
				return $booking;
			}	
		}
		
		return null;
	}
}

?>